@@ -1,5 +1,5 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "copilot" "20260226.819"
+(define-package "copilot" "20260302.1754"
   "An Emacs plugin for GitHub Copilot."
   '((emacs         "27.2")
     (editorconfig  "0.8.2")
@@ -7,10 +7,8 @@
     (compat        "30")
     (track-changes "1.4"))
   :url "https://github.com/copilot-emacs/copilot.el"
-  :commit "7904f13d52e8dcda3af67142c42fa7c8345ba6fe"
-  :revdesc "7904f13d52e8"
+  :commit "fce36db5091260c47c62a70105ba536c5106776f"
+  :revdesc "fce36db50912"
   :keywords '("convenience" "copilot")
   :authors '(("zerol" . "z@zerol.me"))
-  :maintainers '(("Jen-Chieh" . "jcs090218@gmail.com")
-                 ("Rakotomandimby Mihamina" . "mihamina.rakotomandimby@rktmb.org")
-                 ("Bozhidar Batsov" . "bozhidar@batsov.dev")))
+  :maintainers '(("Bozhidar Batsov" . "bozhidar@batsov.dev")))
